// motorzx.h

// Tabla de colores para un nico valor de papel/tinta
// - TINTA -
// 
// NEGRO	0	00000000
// AZUL		1	00000001
// ROJO		2	00000010
// MAGENTA	3	00000011
// VERDE	4	00000100
// AZULADO	5	00000101
// AMARILLO	6	00000110
// BLANCO	7	00000111
// 
// - PAPEL -
//
// NEGRO	0	00000000
// AZUL		8	00001000
// ROJO		16	00010000
// MAGENTA	24	00011000
// VERDE	32	00100000
// AZULADO	40	00101000
// AMARILLO	48	00110000
// BLANCO	56	00111000

// Funciones:

// ___________________________________________
// Borra la pantalla a un color
// Usar como color la suma de valores de la tabla
// que hay al principio
// ___________________________________________

void cls (int color)
{
	#asm
		ld hl,2			;pasamos la variable de entrada al acumulador
		add hl,sp
		ld a, (hl)
		
		ld hl, 22528
		ld de, 22529
		ld bc, 767
		ld (hl), a
		ldir
		ld hl, 16384
		ld de, 16385
		ld bc, 6143
		ld (hl),l
		ldir
	#endasm
}

// ___________________________________________
// Hace un FADE de la pantalla a negro
// Rutina original de Santiago Romero
// https://magazinezx.speccy.org/17/ensamblador.html
// ___________________________________________

void fade ()
{
#asm

   LD B, 16

.fadescreen_loop1
   LD HL, 16384+6144
   LD DE, 768

.fadescreen_loop2
   LD A, (HL)                  ; get attr

   LD C, A
   AND 7                       ; get last  bits
   JR Z, fadescreen_ink_zero   ; if its zero, not dec it

   DEC A
   EX AF, AF

.fadescreen_ink_zero
   LD A, C
   AND 7
   SRA A
   SRA A
   SRA A
   JR Z, fadescreen_paper_zero   ; if its zero, not dec it

   DEC A

.fadescreen_paper_zero
   SLA A
   SLA A
   SLA A
   LD C, A
   EX AF, AF
   ADD A, C

   LD (HL), A
   INC HL

   DEC DE                       ; internal loop
   LD A, D
   OR E
   JP NZ, fadescreen_loop2

   DJNZ fadescreen_loop1

#endasm
}

// ___________________________________________
// Delay, produce una pausa en el programa
// ___________________________________________

void delay (int delay)
{
	#asm
	ld hl,2
	add hl,sp
	ld c, 0
	ld b, 2
	uno:
	ld e, (hl)	;operando base
	ld d, 0
	dos:
	dec de
	ld a,d
	or e
	jp nz,dos
	dec bc
	ld a,b
	or c
	jp nz,uno
	ret
	#endasm
}

// ___________________________________________
// Funcin para sacar un dato por un puerto
// ___________________________________________

void port_out (int port, int value)
{
	#asm
	ld hl,2
	add hl,sp
	ld a, (hl)
	inc hl
	inc hl
	ld c, (hl)
	inc hl
	ld b, (hl)
	out (c),a
	#endasm
}

// ___________________________________________
// Funcin para leer un puerto
// ___________________________________________

int __FASTCALL__ port_in (int port)
{
	#asm
        	ld      b, h
        	ld      c, l
        	ld      h, 0
        	in      l, (c)
	#endasm
}


// ___________________________________________
// Espera interrupcin y para la CPU
// ___________________________________________

void wait_int (void)
{
	#asm
		halt
	#endasm
}

// ___________________________________________
// Posiciona un Sprite de 16x16 a color
// ___________________________________________

void put_sprite_x16 (unsigned char *posicion, unsigned int x, unsigned int y)
{
	// -------------------------------------------
	// RUTINA DE IMPRESION DE UN SPRITE 16x16 PIXELS
	// CON ATRIBUTOS EN CUALQUIER POSICION DE CARACTER
	// ENTRADAS:
	// D ser la posicin del cursor vertical en caracteres
	// E ser la posicin del cursor horizontal en caracteres
	// HL es la posicin de memoria donde tenemos el sprite
	// SALIDAS: se escribe en el mapa de pantalla
	// ADVERTENCIAS: no comprueba lmites de pantalla
	// -------------------------------------------
	#asm
		ld hl,2			;pasamos la variable de entrada al acumulador
		add hl,sp
		ld d, (hl)
		inc hl
		inc hl
		ld e, (hl)
		inc hl
		inc hl
		ld a, (hl)
	        inc hl
	        ld h, (hl)
	        ld l, a
		ld a, d		; recuperamos el valor vertical
		and 7		; nos quedamos con la posicin en el tercio
		rrca
	        rrca
	        rrca		; rotamos para dejar su valor en mltiplos de 32 (linea)
		and 224		; borramos el resto de bits por si las moscas
		or e		; sumamos el valor horizontal
		ld e, a		; e preparado
		ld a, d
		and 24		; modificamos segn el tercio de pantalla
		or 64		; nos posicionamos a partir de 16384 (16384=64+0 en dos bytes)
		ld d, a		; d preparado, ya tenemos la posicin en pantalla
		push de		; guardamos DE (la posicin de pantalla)
		push de
		ld b, 8
		ld c,255	; cargamos C con 255 para no afectar B con LDI
		call draw
		pop de		; recuperamos DE
		ld a,e		; incrementamos una lnea de caracteres (+32 bytes)
		add a,32
		ld e,a		; aqu finaliza la suma de 32 a E
		jr nc, salto
		ld a,d
		add a,8
		ld d,a
		.salto		; aqu finaliza la suma de 32 a D
		ld b, 8
		ld c,255
		call draw
				; Ahora imprimimos los atributos
		pop de		; recuperamos el valor horizontal
		ld a,d		; calculamos el valor de posicin en la pantalla
	        rra
	        rra
	        rra		; multiplicamos por 32
	        and 3		; nos quedamos con los tres bits bajos
	        or 88		; apuntamos al comienzo del mapa de atributos
	        ld d,a          ; ya tenemos d listo, e no hay que cambiarlo
		push de		; guardamos la posicin en pantalla
		ldi		; imprimimos los colores de arriba
		ldi
		pop de		; recuperamos la posicin de pantalla
		ld a,e		; incrementamos una lnea de caracteres (+32 bytes)
		add a,32
		ld e,a		; aqu finaliza la suma de 32 a DE
		ld a,d
		adc a,0
		ld d,a
		ldi		; ponemos los colores de abajo
		ldi
		ret
	
		.draw
		ldi
		ldi
		dec de
		dec de
		inc d	
		djnz draw	; decrementa B y si es cero deja de saltar a draw
		ret
			
	#endasm
}

put_partial1h_sprite_x16 (unsigned char *posicion, unsigned int x, unsigned int y)
{
	#asm
		ld hl,2			;pasamos la variable de entrada al acumulador
		add hl,sp
		ld d, (hl)
		inc hl
		inc hl
		ld e, (hl)
		inc hl
		inc hl
		ld a, (hl)
	        inc hl
	        ld h, (hl)
	        ld l, a
		ld a, d		; recuperamos el valor vertical
		and 7		; nos quedamos con la posicin en el tercio
		rrca
	        rrca
	        rrca		; rotamos para dejar su valor en mltiplos de 32 (linea)
		and 224		; borramos el resto de bits por si las moscas
		or e		; sumamos el valor horizontal
		ld e, a		; e preparado
		ld a, d
		and 24		; modificamos segn el tercio de pantalla
		or 64		; nos posicionamos a partir de 16384 (16384=64+0 en dos bytes)
		ld d, a		; d preparado, ya tenemos la posicin en pantalla
		push de		; guardamos DE (la posicin de pantalla)
		push de
		ld b, 8
		ld c,255	; cargamos C con 255 para no afectar B con LDI
		call draw21
		pop de		; recuperamos DE
		ld a,e		; incrementamos una lnea de caracteres (+32 bytes)
		add a,32
		ld e,a		; aqu finaliza la suma de 32 a E
		jr nc, salto21
		ld a,d
		add a,8
		ld d,a
		.salto21		; aqu finaliza la suma de 32 a D
		ld b, 8
		ld c,255
		call draw21
				; Ahora imprimimos los atributos
		pop de		; recuperamos el valor horizontal
		ld a,d		; calculamos el valor de posicin en la pantalla
	        rra
	        rra
	        rra		; multiplicamos por 32
	        and 3		; nos quedamos con los tres bits bajos
	        or 88		; apuntamos al comienzo del mapa de atributos
	        ld d,a          ; ya tenemos d listo, e no hay que cambiarlo
		push de		; guardamos la posicin en pantalla
		ldi		; imprimimos los colores de arriba
		pop de		; recuperamos la posicin de pantalla
		ld a,e		; incrementamos una lnea de caracteres (+32 bytes)
		add a,32
		ld e,a		; aqu finaliza la suma de 32 a DE
		ld a,d
		adc a,0
		ld d,a
		ldi		; ponemos los colores de abajo
		ret
	
		.draw21
		ldi
		dec de
		inc hl
		inc d	
		djnz draw21	; decrementa B y si es cero deja de saltar a draw
		ret
			
	#endasm
}

put_partial2h_sprite_x16 (unsigned char *posicion, unsigned int x, unsigned int y)
{
	#asm
		ld hl,2			;pasamos la variable de entrada al acumulador
		add hl,sp
		ld d, (hl)
		inc hl
		inc hl
		ld e, (hl)
		inc hl
		inc hl
		ld a, (hl)
	        inc hl
	        ld h, (hl)
	        ld l, a
		ld a, d		; recuperamos el valor vertical
		and 7		; nos quedamos con la posicin en el tercio
		rrca
	        rrca
	        rrca		; rotamos para dejar su valor en mltiplos de 32 (linea)
		and 224		; borramos el resto de bits por si las moscas
		or e		; sumamos el valor horizontal
		ld e, a		; e preparado
		ld a, d
		and 24		; modificamos segn el tercio de pantalla
		or 64		; nos posicionamos a partir de 16384 (16384=64+0 en dos bytes)
		ld d, a		; d preparado, ya tenemos la posicin en pantalla
		push de		; guardamos DE (la posicin de pantalla)
		push de
		ld b, 8
		ld c,255	; cargamos C con 255 para no afectar B con LDI
		call draw22
		pop de		; recuperamos DE
		ld a,e		; incrementamos una lnea de caracteres (+32 bytes)
		add a,32
		ld e,a		; aqu finaliza la suma de 32 a E
		jr nc, salto22
		ld a,d
		add a,8
		ld d,a
		.salto22		; aqu finaliza la suma de 32 a D
		ld b, 8
		ld c,255
		call draw22
				; Ahora imprimimos los atributos
		pop de		; recuperamos el valor horizontal
		ld a,d		; calculamos el valor de posicin en la pantalla
	        rra
	        rra
	        rra		; multiplicamos por 32
	        and 3		; nos quedamos con los tres bits bajos
	        or 88		; apuntamos al comienzo del mapa de atributos
	        ld d,a          ; ya tenemos d listo, e no hay que cambiarlo
		push de		; guardamos la posicin en pantalla
		inc hl		; imprimimos los colores de arriba
		ldi
		inc de
		pop de		; recuperamos la posicin de pantalla
		ld a,e		; incrementamos una lnea de caracteres (+32 bytes)
		add a,32
		ld e,a		; aqu finaliza la suma de 32 a DE
		ld a,d
		adc a,0
		ld d,a
		inc hl		; ponemos los colores de abajo
		ldi
		ret
	
		.draw22
		inc hl
		ldi
		dec de
		inc d	
		djnz draw22	; decrementa B y si es cero deja de saltar a draw
		ret
			
	#endasm
}

put_partial1v_sprite_x16 (unsigned char *posicion, unsigned int x, unsigned int y)
{
	#asm
		ld hl,2			;pasamos la variable de entrada al acumulador
		add hl,sp
		ld d, (hl)
		inc hl
		inc hl
		ld e, (hl)
		inc hl
		inc hl
		ld a, (hl)
	        inc hl
	        ld h, (hl)
	        ld l, a
		ld a, d		; recuperamos el valor vertical
		and 7		; nos quedamos con la posicin en el tercio
		rrca
	        rrca
	        rrca		; rotamos para dejar su valor en mltiplos de 32 (linea)
		and 224		; borramos el resto de bits por si las moscas
		or e		; sumamos el valor horizontal
		ld e, a		; e preparado
		ld a, d
		and 24		; modificamos segn el tercio de pantalla
		or 64		; nos posicionamos a partir de 16384 (16384=64+0 en dos bytes)
		ld d, a		; d preparado, ya tenemos la posicin en pantalla
		push de		; guardamos DE (la posicin de pantalla)
		ld b, 8
		ld c,255	; cargamos C con 255 para no afectar B con LDI
		call draw23
		ld b, 8
		ld c,255	; cargamos C con 255 para no afectar B con LDI
		.lele23
		inc hl
		inc hl
		djnz lele23
				; Ahora imprimimos los atributos
		pop de		; recuperamos el valor horizontal
		ld a,d		; calculamos el valor de posicin en la pantalla
	        rra
	        rra
	        rra		; multiplicamos por 32
	        and 3		; nos quedamos con los tres bits bajos
	        or 88		; apuntamos al comienzo del mapa de atributos
	        ld d,a          ; ya tenemos d listo, e no hay que cambiarlo
		ldi		; imprimimos los colores de arriba
		ldi
		ret
	
		.draw23
		ldi
		ldi
		dec de
		dec de
		inc d	
		djnz draw23	; decrementa B y si es cero deja de saltar a draw
		ret
			
	#endasm
}

put_partial2v_sprite_x16 (unsigned char *posicion, unsigned int x, unsigned int y)
{
	#asm
		ld hl,2			;pasamos la variable de entrada al acumulador
		add hl,sp
		ld d, (hl)
		inc hl
		inc hl
		ld e, (hl)
		inc hl
		inc hl
		ld a, (hl)
	        inc hl
	        ld h, (hl)
	        ld l, a
		ld a, d		; recuperamos el valor vertical
		and 7		; nos quedamos con la posicin en el tercio
		rrca
	        rrca
	        rrca		; rotamos para dejar su valor en mltiplos de 32 (linea)
		and 224		; borramos el resto de bits por si las moscas
		or e		; sumamos el valor horizontal
		ld e, a		; e preparado
		ld a, d
		and 24		; modificamos segn el tercio de pantalla
		or 64		; nos posicionamos a partir de 16384 (16384=64+0 en dos bytes)
		ld d, a		; d preparado, ya tenemos la posicin en pantalla
		push de		; guardamos DE (la posicin de pantalla)
		ld b, 8
		ld c,255	; cargamos C con 255 para no afectar B con LDI
		.lele24
		inc hl
		inc hl
		djnz lele24
		ld b, 8
		ld c,255	; cargamos C con 255 para no afectar B con LDI
		call draw24
				; Ahora imprimimos los atributos
		pop de		; recuperamos el valor horizontal
		ld a,d		; calculamos el valor de posicin en la pantalla
	        rra
	        rra
	        rra		; multiplicamos por 32
	        and 3		; nos quedamos con los tres bits bajos
	        or 88		; apuntamos al comienzo del mapa de atributos
	        ld d,a          ; ya tenemos d listo, e no hay que cambiarlo
		inc hl
		inc hl
		ldi		; imprimimos los colores de abajo
		ldi
		ret
	
		.draw24
		ldi
		ldi
		dec de
		dec de
		inc d	
		djnz draw24	; decrementa B y si es cero deja de saltar a draw
		ret
			
	#endasm
}

// ___________________________________________
// Posiciona una letra de 8x8 sin color
// ___________________________________________

void put_char (unsigned char *posicion, unsigned int x, unsigned int y)
{
   // -------------------------------------------
   // RUTINA DE IMPRESION DE UNA LETRA DE 8x8 PIXELS
   // EN CUALQUIER POSICION DE CARACTER
   // ENTRADAS:
   // D ser la posicin del cursor vertical en caracteres
   // E ser la posicin del cursor horizontal en caracteres
   // HL es la posicin de memoria donde tenemos la letra
   // SALIDAS: se escribe en el mapa de pantalla
   // ADVERTENCIAS: no comprueba lmites de pantalla
   // -------------------------------------------
   #asm
      ld hl,2         ;pasamos la variable de entrada al acumulador
      add hl,sp
      ld d, (hl)
      inc hl
      inc hl
      ld e, (hl)
      inc hl
      inc hl
      ld a, (hl)
           inc hl
           ld h, (hl)
           ld l, a
      ld a, d      ; recuperamos el valor vertical
      and 7      ; nos quedamos con la posicin en el tercio
      rrca
           rrca
           rrca      ; rotamos para dejar su valor en mltiplos de 32 (linea)
      and 224      ; borramos el resto de bits por si las moscas
      or e      ; sumamos el valor horizontal
      ld e, a      ; e preparado
      ld a, d
      and 24      ; modificamos segn el tercio de pantalla
      or 64      ; nos posicionamos a partir de 16384 (16384=64+0 en dos bytes)
      ld d, a      ; d preparado, ya tenemos la posicin en pantalla
      ld b, 8
      ld c,255   ; cargamos C con 255 para no afectar B con LDI
      .draw26
      ldi
      dec de
      inc d   
      djnz draw26   ; decrementa B y si es cero deja de saltar a draw
      ret
         
   #endasm
}

